package calculus;

import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;
import java.util.TreeSet;

import algebra.numbers.Rational;
import algebra.words.Word;

/**
 * @author Anton Mellit
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
class SplitComp implements Comparator {
	/**
	 * @see java.util.Comparator#compare(Object, Object)
	 */
	public int compare(Object o1, Object o2) {
		Word.Split s1 = (Word.Split) o1;
		Word.Split s2 = (Word.Split) o2;
		int c = s1.left.compareTo(s2.left);
		if(c!=0) return c;
		c = s1.center.compareTo(s2.center);
		if(c!=0) return c;
		c = s1.right.compareTo(s2.right);
		return c;
	}

}

class Composition {
	public WordSum s1;
	public WordSum s2;
	public Word.Split split;
}

public class Groebner {
	protected Collection relations;
	protected Set splitsUsed;
	protected Collection newRelations;
	protected Collection newCompositions;
	protected boolean dump;
	
	public Groebner() {
		relations = new LinkedList();
		splitsUsed = new TreeSet(new SplitComp());
		newRelations = new LinkedList();
		newCompositions = new LinkedList();
	}
	
	
	/**
	 * Sets the dump.
	 * @param dump The dump to set
	 */
	public void setDump(boolean dump) {
		this.dump = dump;
	}
	
	public void addRelation(WordSum rel) {
		rel.reduce(relations);
		if(!rel.isZero()) {
			rel.normalize();
			newRelations.add(rel);
			if(dump) {
//				System.out.println("Added new relation "+rel);
			}
		}
	}
	
	protected boolean pairwiseReduce() {
		if(dump) {
			System.out.println("Pairwise reduce ("+relations.size()+" relations)");
		}
		boolean changed = false;
		int k = 0;
		for(Iterator i1 = relations.iterator(); i1.hasNext(); ) {
			k++;
			WordSum s1 = (WordSum) i1.next();
			if(s1.isZero()) {
				i1.remove();
				changed = true;
			} else {			
				for(Iterator i2 = relations.iterator(); i2.hasNext(); ) {
					WordSum s2 = (WordSum) i2.next();
					if(s1!=s2) {
						if(s2.reduce(s1)) {
							s2.normalize();
							changed = true;
						}
					}
				}
			}
			if(k%10==0) {
				System.out.println("Ready: "+k);
			}
		}
		return changed;
	}
	
	protected void fullReduce() {
		if(dump) {
			System.out.println("Reducing all relations");
		}
		while(pairwiseReduce());
	}
	
	public void dumpStatus() {
		if(dump) {
			System.out.println("Have "+relations.size()+" relations:");
			for(Iterator i1 = relations.iterator(); i1.hasNext(); ) {
				WordSum s1 = (WordSum) i1.next();
				System.out.println(s1.toString(true));
			}
		}
	}
	
	public void mergeRelations() {
		relations.addAll(newRelations);
		newRelations.clear();
		fullReduce();
		dumpStatus();
	}
	
	protected void makeCompositions() {
		for(Iterator i1 = relations.iterator(); i1.hasNext(); ) {
			WordSum s1 = (WordSum) i1.next();
			Word m1 = (Word) s1.getMainObject();
			for(Iterator i2 = relations.iterator(); i2.hasNext(); ) {
				WordSum s2 = (WordSum) i2.next();
				Word m2 = (Word) s2.getMainObject();
				Collection splits = m1.getAllSplits(m2);
				for(Iterator i3 = splits.iterator(); i3.hasNext(); ) {
					Word.Split split = (Word.Split) i3.next();
					if(!splitsUsed.contains(split)) {
						splitsUsed.add(split);
						addComposition(s1, s2, split);
					}
				}
			}
		}
	}
	
	protected void addComposition(WordSum s1, WordSum s2, Word.Split split) {
		if(dump) {
			System.out.println("Scheduled composition "+splitToString(split));
		}
		Composition c = new Composition();
		c.s1 = s1;
		c.s2 = s2;
		c.split = split;
		newCompositions.add(c);
	}
	
	protected String splitToString(Word.Split split) {
		return split.left.toString()+"|"+split.center+"|"+split.right;
	}
		
	protected void mergeCompositions() {
		int j = 0;
		for(Iterator i = newCompositions.iterator(); i.hasNext(); ) {
			mergeComposition((Composition) i.next(), j);
			j++;
		}
		newCompositions.clear();
	}
	
	protected void mergeComposition(Composition c, int i) {
		if(dump) {
			System.out.println("Making composition "+splitToString(c.split)+" ("+i+"/"+newCompositions.size()+")");
		}
		WordSum s = new WordSum();
		s.add(c.s1, Word.ONE, c.split.right, Rational.valueOf(1));
		s.add(c.s2, c.split.left, Word.ONE, Rational.valueOf(-1));
		addRelation(s);
	}
	
	public boolean fullCompose() {
		makeCompositions();
		boolean changed = !newCompositions.isEmpty();
		mergeCompositions();
		return changed;
	}
	
	public void groebnerClosure() {
		boolean changed;
		do {
			changed = fullCompose();
			mergeRelations();
		} while(changed);
	}
	
	public Collection getRelations() {
		return relations;
	}
	
	public void dump() {
		System.out.println("Relations:");
		for (Iterator iter = relations.iterator(); iter.hasNext();) {
			WordSum ws = (WordSum) iter.next();
			System.out.println(ws);
		}
	}
}
