package util;

import java.io.*;

import algebra.words.Letter;
import algebra.words.Word;
import calculus.CenterAnalysis;
import calculus.WordSum;
/**
 * @author Anton Mellit
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class GroebnerBasis {

	public static void main(String[] args) {
//		args = new String[]{"groebner", "test.in", "test.out", "t"};
		if(args.length<2) {
			System.out.println("Please, specify command followed by setting file");
			System.out.println("Possible commands are: groebner, center, calculator");
			return;
		}
		Setting set = new Setting();
		try {
			set.load(args[1]);
			set.dump();

			String command = args[0];
			set.getRelations().setDump(true);
			if(command.equalsIgnoreCase("center")) {
				if(args.length<4) {
					System.out.println("Please, specify output file and letter which will be assigned result value");
					return;
				}
				String letterName = args[3];
				Letter centralLetter = set.getAbc().getFirstLetter(letterName);
				if(centralLetter==null) {
					System.out.println("Error: Letter "+letterName+" is not in the alphabet");
					return;
				}
				CenterAnalysis ca = new CenterAnalysis(set.getGenerators(), set.getAbc().getLetters(), set.getRelations().getRelations());
				ca.setDump(true);
				WordSum ws = ca.findCenter();
				if(ws==null) {
					System.out.println("Center not found");
				} else {
					System.out.println("Central element found and is :");
					System.out.println(ws);
					ws.add(new Word(centralLetter), -1);
					set.getRelations().addRelation(ws);
					set.getRelations().mergeRelations();
				}
				set.save(args[2]);
			} else if(command.equalsIgnoreCase("groebner")) {
				if(args.length<3) {
					System.out.println("Please, specify output file");
					return;
				}
				set.getRelations().setDump(true);
				set.getRelations().groebnerClosure();
				set.save(args[2]);
			} else if(command.equalsIgnoreCase("calculator")) {
				System.out.println("Ready for calculations, you may enter statements, type exit to exit");
				BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
				try {
					while(true) {
						String line = reader.readLine();
						if(line.equalsIgnoreCase("exit")) {
							break;
						}
						WordSum res;
						try {
							res = WordSum.parse(line, set.getAbc());
							res.reduce(set.getRelations().getRelations());
						}catch(Exception e) {
							System.out.println("Your input is incorrect, please try again");
							continue;
						}
						System.out.println(res);
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			} else {
				System.out.println("Command "+command+" not supported");
				return;
			}			
		} catch(IOException e) {
			System.out.println(e.getLocalizedMessage());
		}
	}
}
